@@ -37,6 +37,8 @@
 class PListNode;
 
 class PList : public RefCounted {
+	GDSOFTCLASS(PList, RefCounted);
+
 	friend class PListNode;
 
 public:
@@ -119,7 +121,4 @@ public:
 	void store_asn1_size(PackedByteArray &p_stream, uint8_t p_len_octets) const;
 	bool store_asn1(PackedByteArray &p_stream, uint8_t p_len_octets) const;
 	void store_text(String &p_stream, uint8_t p_indent) const;
-
-	PListNode() {}
-	~PListNode() {}
 };
