@@ -469,7 +469,6 @@ static const char *language_list[][2] = {
 	{ "chy", "Cheyenne" },
 	{ "cic", "Chickasaw" },
 	{ "ckb", "Central Kurdish" },
-	{ "csb", "Kashubian" },
 	{ "cmn", "Mandarin Chinese" },
 	{ "co", "Corsican" },
 	{ "cop", "Coptic" },
@@ -662,7 +661,6 @@ static const char *language_list[][2] = {
 	{ "kv", "Komi" },
 	{ "kw", "Cornish" },
 	{ "ky", "Kirghiz" },
-	{ "lag", "Langi" },
 	{ "la", "Latin" },
 	{ "lad", "Ladino" },
 	{ "lag", "Langi" },
@@ -982,7 +980,630 @@ static const char *language_list[][2] = {
 	{ "zu", "Zulu" },
 	{ "zun", "Zuni" },
 	{ "zza", "Zaza" },
-	{ nullptr, nullptr }
+	{ nullptr, nullptr },
+};
+
+// Scripts used by languages.
+// Reference:
+// - https://www.unicode.org/cldr/charts/48/supplemental/languages_and_scripts.html
+
+static const char *language_script_list[][2] = {
+	{ "aa", "Latn" },
+	{ "ab", "Cyrl" },
+	{ "ace", "Latn" },
+	{ "ach", "Latn" },
+	{ "ada", "Latn" },
+	{ "ady", "Cyrl" },
+	{ "ae", "Avst" },
+	{ "aeb", "Arab" },
+	{ "af", "Latn" },
+	{ "afh", "Latn" },
+	{ "agq", "Latn" },
+	{ "ain", "Kana Latn" },
+	{ "agr", "Latn" },
+	{ "ak", "Latn" },
+	{ "akk", "Xsux" },
+	{ "akz", "Latn" },
+	{ "ale", "Latn" },
+	{ "aln", "Latn" },
+	{ "alt", "Cyrl" },
+	{ "am", "Ethi" },
+	{ "an", "Latn" },
+	{ "ang", "Latn" },
+	{ "anp", "Deva" },
+	{ "ar", "Arab Syrc" },
+	{ "arc", "Armi Nbat Palm" },
+	{ "arn", "Latn" },
+	{ "aro", "Latn" },
+	{ "arp", "Latn" },
+	{ "arq", "Arab" },
+	{ "ars", "Arab" },
+	{ "arw", "Latn" },
+	{ "ary", "Arab" },
+	{ "arz", "Arab" },
+	{ "as", "Beng" },
+	{ "asa", "Latn" },
+	{ "ast", "Latn" },
+	{ "av", "Cyrl" },
+	{ "avk", "Latn" },
+	{ "awa", "Deva" },
+	{ "ayc", "Latn" },
+	{ "ay", "Latn" },
+	{ "az", "Latn Arab Cyrl" },
+	{ "ba", "Cyrl" },
+	{ "bal", "Arab Latn" },
+	{ "ban", "Latn Bali" },
+	{ "bar", "Latn" },
+	{ "bas", "Latn" },
+	{ "bax", "Bamu" },
+	{ "bbc", "Latn Batk" },
+	{ "bbj", "Latn" },
+	{ "be", "Cyrl" },
+	{ "bej", "Arab" },
+	{ "bem", "Latn" },
+	{ "ber", "Latn Arab Tfng" },
+	{ "bew", "Latn" },
+	{ "bez", "Latn" },
+	{ "bfd", "Latn" },
+	{ "bfq", "Taml" },
+	{ "bg", "Cyrl" },
+	{ "bhb", "Deva" },
+	{ "bgn", "Arab" },
+	{ "bho", "Deva" },
+	{ "bi", "Latn" },
+	{ "bik", "Latn" },
+	{ "bin", "Latn" },
+	{ "bjn", "Latn" },
+	{ "bkm", "Latn" },
+	{ "bla", "Latn" },
+	{ "bm", "Latn Nkoo" },
+	{ "bn", "Beng" },
+	{ "bo", "Tibt" },
+	{ "bpy", "Beng" },
+	{ "bqi", "Arab" },
+	{ "br", "Latn" },
+	{ "brh", "Arab Latn" },
+	{ "brx", "Deva" },
+	{ "bs", "Latn Cyrl" },
+	{ "bss", "Latn" },
+	{ "bua", "Cyrl" },
+	{ "bug", "Latn Bug" },
+	{ "bum", "Latn" },
+	{ "byn", "Ethi" },
+	{ "byv", "Latn" },
+	{ "ca", "Latn" },
+	{ "cad", "Latn" },
+	{ "car", "Latn" },
+	{ "cay", "Latn" },
+	{ "cch", "Latn" },
+	{ "ccp", "Cakm Beng" },
+	{ "ce", "Cyrl" },
+	{ "ceb", "Latn" },
+	{ "cgg", "Latn" },
+	{ "ch", "Latn" },
+	{ "chb", "Latn" },
+	{ "chg", "Arab" },
+	{ "chk", "Latn" },
+	{ "chm", "Cyrl" },
+	{ "chn", "Latn" },
+	{ "cho", "Latn" },
+	{ "chp", "Latn Cans" },
+	{ "chr", "Cher" },
+	{ "chy", "Latn" },
+	{ "cic", "Latn" },
+	{ "ckb", "Arab" },
+	{ "cmn", "Hant Hans Hani" },
+	{ "co", "Latn" },
+	{ "cop", "Copt Arab Grek" },
+	{ "cps", "Latn" },
+	{ "cr", "Cans Latn" },
+	{ "crh", "Cyrl" },
+	{ "crs", "Latn" },
+	{ "cs", "Latn" },
+	{ "csb", "Latn" },
+	{ "cu", "Cyrl" },
+	{ "cv", "Cyrl" },
+	{ "cy", "Latn" },
+	{ "da", "Latn" },
+	{ "dak", "Latn" },
+	{ "dar", "Cyrl" },
+	{ "dav", "Latn" },
+	{ "de", "Latn Runr" },
+	{ "del", "Latn" },
+	{ "den", "Latn Cans" },
+	{ "dgr", "Latn" },
+	{ "din", "Latn" },
+	{ "dje", "Latn" },
+	{ "doi", "Deva Arab Takr" },
+	{ "dsb", "Latn" },
+	{ "dtp", "Latn" },
+	{ "dua", "Latn" },
+	{ "dum", "Latn" },
+	{ "dv", "Thaa" },
+	{ "dyo", "Latn Arab" },
+	{ "dyu", "Latn" },
+	{ "dz", "Tibt" },
+	{ "dzg", "Latn" },
+	{ "ebu", "Latn" },
+	{ "ee", "Latn" },
+	{ "efi", "Latn" },
+	{ "egl", "Latn" },
+	{ "egy", "Egyp" },
+	{ "eka", "Latn" },
+	{ "el", "Grek" },
+	{ "elx", "Xsux" },
+	{ "en", "Latn Dsrt Shaw" },
+	{ "enm", "Latn" },
+	{ "eo", "Latn" },
+	{ "es", "Latn" },
+	{ "esu", "Latn" },
+	{ "et", "Latn" },
+	{ "eu", "Latn" },
+	{ "ewo", "Latn" },
+	{ "ext", "Latn" },
+	{ "fa", "Arab" },
+	{ "fan", "Latn" },
+	{ "fat", "Latn" },
+	{ "ff", "Latn Adlm" },
+	{ "fi", "Latn" },
+	{ "fil", "Latn Tglg" },
+	{ "fit", "Latn" },
+	{ "fj", "Latn" },
+	{ "fo", "Latn" },
+	{ "fon", "Latn" },
+	{ "fr", "Latn Dupl" },
+	{ "frc", "Latn" },
+	{ "frm", "Latn" },
+	{ "fro", "Latn" },
+	{ "frp", "Latn" },
+	{ "frr", "Latn" },
+	{ "frs", "Latn" },
+	{ "fur", "Latn" },
+	{ "fy", "Latn" },
+	{ "ga", "Latn" },
+	{ "gaa", "Latn" },
+	{ "gag", "Latn Cyrl" },
+	{ "gan", "Hans Hani" },
+	{ "gay", "Latn" },
+	{ "gba", "Latn" },
+	{ "gbz", "Arab" },
+	{ "gd", "Latn" },
+	{ "gez", "Ethi" },
+	{ "gil", "Latn" },
+	{ "gl", "Latn" },
+	{ "glk", "Arab" },
+	{ "gmh", "Latn" },
+	{ "gn", "Latn" },
+	{ "goh", "Latn" },
+	{ "gom", "Deav Latn Knda Mlym Arab" },
+	{ "gon", "Deva Telu" },
+	{ "gor", "Latn" },
+	{ "got", "Goth" },
+	{ "grb", "Latn" },
+	{ "grc", "Grek" },
+	{ "gsw", "Latn" },
+	{ "gu", "Gujr" },
+	{ "guc", "Latn" },
+	{ "gur", "Latn" },
+	{ "guz", "Latn" },
+	{ "gv", "Latn" },
+	{ "gwi", "Latn" },
+	{ "ha", "Latn Arab" },
+	{ "hai", "Latn" },
+	{ "hak", "Hans Hant Hani" },
+	{ "haw", "Latn" },
+	{ "he", "Hebr" },
+	{ "hi", "Deva Latn Mahj" },
+	{ "hif", "Deva Latn" },
+	{ "hil", "Latn" },
+	{ "hit", "Xsux" },
+	{ "hmn", "Latn Hmng" },
+	{ "ho", "Latn" },
+	{ "hne", "Deva" },
+	{ "hr", "Latn" },
+	{ "hsb", "Latn" },
+	{ "hsn", "Hans Hani" },
+	{ "ht", "Latn" },
+	{ "hu", "Latn" },
+	{ "hup", "Latn" },
+	{ "hus", "Latn" },
+	{ "hy", "Armn" },
+	{ "hz", "Latn" },
+	{ "ia", "Latn" },
+	{ "iba", "Latn" },
+	{ "ibb", "Latn" },
+	{ "id", "Latn Arab" },
+	{ "ie", "Latn" },
+	{ "ig", "Latn" },
+	{ "ii", "Yiii Latn" },
+	{ "ik", "Latn" },
+	{ "ilo", "Latn" },
+	{ "inh", "Cyrl" },
+	{ "io", "Latn" },
+	{ "is", "Latn" },
+	{ "it", "Latn" },
+	{ "iu", "Cans Latn" },
+	{ "izh", "Latn" },
+	{ "ja", "Jpan Hani Hira Kana" },
+	{ "jam", "Latn" },
+	{ "jbo", "Latn" },
+	{ "jgo", "Latn" },
+	{ "jmc", "Latn" },
+	{ "jpr", "Hebr" },
+	{ "jrb", "Hebr" },
+	{ "jut", "Latn" },
+	{ "jv", "Latn Java" },
+	{ "ka", "Geor" },
+	{ "kaa", "Cyrl Latn" },
+	{ "kab", "Latn" },
+	{ "kac", "Latn" },
+	{ "kaj", "Latn" },
+	{ "kam", "Latn" },
+	{ "kaw", "Bali Java Kawi" },
+	{ "kbd", "Cyrl" },
+	{ "kbl", "Arab" },
+	{ "kcg", "Latn" },
+	{ "kde", "Latn" },
+	{ "kea", "Latn" },
+	{ "ken", "Latn" },
+	{ "kfo", "Latn" },
+	{ "kg", "Latn" },
+	{ "kgp", "Latn" },
+	{ "kha", "Latn Beng" },
+	{ "kho", "Brah Khar" },
+	{ "khq", "Latn" },
+	{ "khw", "Arab" },
+	{ "ki", "Latn" },
+	{ "kiu", "Latn" },
+	{ "kj", "Latn" },
+	{ "kk", "Cyrl Arab" },
+	{ "kkj", "Latn" },
+	{ "kl", "Latn" },
+	{ "kln", "Latn" },
+	{ "km", "Khmr" },
+	{ "kmb", "Latn" },
+	{ "kn", "Knda" },
+	{ "ko", "Kore Hang Hani" },
+	{ "koi", "Cyrl" },
+	{ "kok", "Deva Latn" },
+	{ "kos", "Latn" },
+	{ "kpe", "Latn" },
+	{ "kr", "Latn" },
+	{ "krc", "Cyrl" },
+	{ "kri", "Latn" },
+	{ "krj", "Latn" },
+	{ "krl", "Latn" },
+	{ "kru", "Deva" },
+	{ "ks", "Arab Deva" },
+	{ "ksb", "Latn" },
+	{ "ksf", "Latn" },
+	{ "ksh", "Latn" },
+	{ "ku", "Latn Arab Cyrl" },
+	{ "kum", "Cyrl" },
+	{ "kut", "Latn" },
+	{ "kv", "Cyrl Perm" },
+	{ "kw", "Latn" },
+	{ "ky", "Cyrl Arab Latn" },
+	{ "la", "Latn" },
+	{ "lad", "Hebr" },
+	{ "lag", "Latn" },
+	{ "lah", "Arab" },
+	{ "lam", "Latn" },
+	{ "lb", "Latn" },
+	{ "lez", "Cyrl Aghb" },
+	{ "lfn", "Latn Cyrl" },
+	{ "lg", "Latn" },
+	{ "li", "Latn" },
+	{ "lij", "Latn" },
+	{ "liv", "Latn" },
+	{ "lkt", "Latn" },
+	{ "lmo", "Latn" },
+	{ "ln", "Latn" },
+	{ "lo", "Laoo" },
+	{ "lol", "Latn" },
+	{ "lou", "Latn" },
+	{ "loz", "Latn" },
+	{ "lrc", "Arab" },
+	{ "lt", "Latn" },
+	{ "ltg", "Latn" },
+	{ "lu", "Latn" },
+	{ "lua", "Latn" },
+	{ "lui", "Latn" },
+	{ "lun", "Latn" },
+	{ "luo", "Latn" },
+	{ "lus", "Beng" },
+	{ "luy", "Latn" },
+	{ "lv", "Latn" },
+	{ "lzh", "Hant" },
+	{ "lzz", "Latn Geor" },
+	{ "mad", "Latn" },
+	{ "maf", "Latn" },
+	{ "mag", "Deva" },
+	{ "mai", "Deva Tirh" },
+	{ "mak", "Latn Bugi" },
+	{ "man", "Latn Nkoo" },
+	{ "mas", "Latn" },
+	{ "mde", "Arab Latn" },
+	{ "mdf", "Cyrl" },
+	{ "mdr", "Latn Bugi" },
+	{ "men", "Latn Mend" },
+	{ "mer", "Latn" },
+	{ "mfe", "Latn" },
+	{ "mg", "Latn" },
+	{ "mga", "Latn" },
+	{ "mgh", "Latn" },
+	{ "mgo", "Latn" },
+	{ "mh", "Latn" },
+	{ "mhr", "Cyrl" },
+	{ "mi", "Latn" },
+	{ "mic", "Latn" },
+	{ "min", "Latn" },
+	{ "miq", "Latn" },
+	{ "mjw", "Latn" },
+	{ "mk", "Cyrl" },
+	{ "ml", "Mlym" },
+	{ "mn", "Cyrl Mong Phag" },
+	{ "mnc", "Mong" },
+	{ "mni", "Beng Mtei" },
+	{ "mnw", "Mymr" },
+	{ "mos", "Latn" },
+	{ "moh", "Latn" },
+	{ "mr", "Deva Modi" },
+	{ "mrj", "Cyrl" },
+	{ "ms", "Latn Arab" },
+	{ "mt", "Latn" },
+	{ "mua", "Latn" },
+	{ "mus", "Latn" },
+	{ "mwl", "Latn" },
+	{ "mwr", "Deva" },
+	{ "mwv", "Latn" },
+	{ "my", "Mymr" },
+	{ "mye", "Latn" },
+	{ "myv", "Cyrl" },
+	{ "mzn", "Arab" },
+	{ "na", "Latn" },
+	{ "nah", "Latn" },
+	{ "nan", "Hans Hant Hani" },
+	{ "nap", "Latn" },
+	{ "naq", "Latn" },
+	{ "nan", "Latn Hans Hant Hani" },
+	{ "nb", "Latn" },
+	{ "nd", "Latn" },
+	{ "nds", "Latn" },
+	{ "ne", "Deva" },
+	{ "new", "Deva" },
+	{ "nhn", "Latn" },
+	{ "ng", "Latn" },
+	{ "nia", "Latn" },
+	{ "niu", "Latn" },
+	{ "njo", "Latn" },
+	{ "nl", "Latn" },
+	{ "nmg", "Latn" },
+	{ "nn", "Latn" },
+	{ "nnh", "Latn" },
+	{ "nog", "Cyrl" },
+	{ "non", "Runr" },
+	{ "nov", "Latn" },
+	{ "nqo", "Nkoo" },
+	{ "nr", "Latn" },
+	{ "nso", "Latn" },
+	{ "nus", "Latn" },
+	{ "nv", "Latn" },
+	{ "nwc", "Ranj Newa Deva Sidd Brah" },
+	{ "ny", "Latn" },
+	{ "nym", "Latn" },
+	{ "nyn", "Latn" },
+	{ "nyo", "Latn" },
+	{ "nzi", "Latn" },
+	{ "oc", "Latn" },
+	{ "oj", "Cans Latn" },
+	{ "om", "Latn Ethi" },
+	{ "or", "Orya" },
+	{ "os", "Cyrl" },
+	{ "osa", "Osge Latn" },
+	{ "ota", "Arab" },
+	{ "pa", "Guru Arab" },
+	{ "pag", "Latn" },
+	{ "pal", "Phli Phlp" },
+	{ "pam", "Latn" },
+	{ "pap", "Latn" },
+	{ "pau", "Latn" },
+	{ "pcd", "Latn" },
+	{ "pcm", "Latn" },
+	{ "pdc", "Latn" },
+	{ "pdt", "Latn" },
+	{ "peo", "Xpeo" },
+	{ "pfl", "Latn" },
+	{ "phn", "Phnx" },
+	{ "pi", "Latn Deva Mymr Sinh Thai" },
+	{ "pl", "Latn" },
+	{ "pms", "Latn" },
+	{ "pnt", "Grek Cyrl Latn" },
+	{ "pon", "Latn" },
+	{ "pr", "Latn" },
+	{ "prg", "Latn" },
+	{ "pro", "Latn" },
+	{ "prs", "Arab" },
+	{ "ps", "Arab" },
+	{ "pt", "Latn" },
+	{ "qu", "Latn" },
+	{ "quc", "Latn" },
+	{ "qug", "Latn" },
+	{ "quy", "Latn" },
+	{ "quz", "Latn" },
+	{ "raj", "Deva" },
+	{ "rap", "Latn" },
+	{ "rar", "Latn" },
+	{ "rgn", "Latn" },
+	{ "rif", "Latn Tfng" },
+	{ "rm", "Latn" },
+	{ "rn", "Latn" },
+	{ "ro", "Latn Cyrl" },
+	{ "rof", "Latn" },
+	{ "rom", "Latn Cyrl" },
+	{ "rtm", "Latn" },
+	{ "ru", "Cyrl" },
+	{ "rue", "Cyrl" },
+	{ "rug", "Latn" },
+	{ "rup", "Latn" },
+	{ "rw", "Latn" },
+	{ "rwk", "Latn" },
+	{ "sa", "Deva Gran Shrd Sidd Sinh" },
+	{ "sad", "Latn" },
+	{ "sah", "Cyrl" },
+	{ "sam", "Samr Hebr" },
+	{ "saq", "Latn" },
+	{ "sas", "Latn" },
+	{ "sat", "Olck Beng Deva Latn Orya" },
+	{ "saz", "Saur" },
+	{ "sba", "Latn" },
+	{ "sbp", "Latn" },
+	{ "sc", "Latn" },
+	{ "scn", "Latn" },
+	{ "sco", "Latn" },
+	{ "sd", "Arab Deva Khoj Sind" },
+	{ "sdc", "Latn" },
+	{ "sdh", "Arab" },
+	{ "se", "Latn Cyrl Arab" },
+	{ "see", "Latn" },
+	{ "seh", "Latn" },
+	{ "sei", "Latn" },
+	{ "sel", "Cyrl" },
+	{ "ses", "Latn" },
+	{ "sg", "Latn" },
+	{ "sga", "Latn Ogam" },
+	{ "sgs", "Latn" },
+	{ "sh", "Latn Cyrl" },
+	{ "shi", "Tfng Latn Arab" },
+	{ "shn", "Mymr" },
+	{ "shs", "Latn Dupl" },
+	{ "shu", "Arab" },
+	{ "si", "Sinh" },
+	{ "sid", "Latn" },
+	{ "sk", "Latn" },
+	{ "sl", "Latn" },
+	{ "sli", "Latn" },
+	{ "sly", "Latn" },
+	{ "sm", "Latn" },
+	{ "sma", "Latn" },
+	{ "smj", "Latn" },
+	{ "smn", "Latn" },
+	{ "sms", "Latn" },
+	{ "sn", "Latn" },
+	{ "snk", "Latn" },
+	{ "so", "Latn Arab Osma" },
+	{ "sog", "Sogd Syrc Mani" },
+	{ "son", "Latn Arab" },
+	{ "sq", "Latn Elba" },
+	{ "sr", "Cyrl Latn" },
+	{ "srn", "Latn" },
+	{ "srr", "Latn" },
+	{ "ss", "Latn" },
+	{ "ssy", "Latn" },
+	{ "st", "Latn" },
+	{ "stq", "Latn" },
+	{ "su", "Latn Sund" },
+	{ "suk", "Latn" },
+	{ "sus", "Latn Arab" },
+	{ "sux", "Xsux" },
+	{ "sv", "Latn" },
+	{ "sw", "Latn" },
+	{ "swb", "Arab Latn" },
+	{ "swc", "Arab Latn" },
+	{ "syc", "Syrc" },
+	{ "syr", "Syrc" },
+	{ "szl", "Latn" },
+	{ "ta", "Taml" },
+	{ "tcy", "Knda" },
+	{ "te", "Telu" },
+	{ "tem", "Latn" },
+	{ "teo", "Latn" },
+	{ "ter", "Latn" },
+	{ "tet", "Latn" },
+	{ "tg", "Cyrl Arab Latn" },
+	{ "th", "Thai" },
+	{ "the", "Deva" },
+	{ "ti", "Ethi" },
+	{ "tig", "Ethi" },
+	{ "tiv", "Latn" },
+	{ "tk", "Latn Arab Cyrl" },
+	{ "tkl", "Latn" },
+	{ "tkr", "Latn Cyrl" },
+	{ "tl", "Latn Arab Blis" },
+	{ "tlh", "Piqd Latn" },
+	{ "tli", "Latn" },
+	{ "tly", "Latn Arab Cyrl" },
+	{ "tmh", "Latn" },
+	{ "tn", "Latn" },
+	{ "to", "Latn" },
+	{ "tog", "Latn" },
+	{ "tpi", "Latn" },
+	{ "tr", "Latn Arab" },
+	{ "tru", "Latn Syrc" },
+	{ "trv", "Latn" },
+	{ "ts", "Latn" },
+	{ "tsd", "Grek" },
+	{ "tsi", "Latn" },
+	{ "tt", "Cyrl" },
+	{ "ttt", "Latn Arab Cyrl" },
+	{ "tum", "Latn" },
+	{ "tvl", "Latn" },
+	{ "tw", "Latn" },
+	{ "twq", "Latn" },
+	{ "ty", "Latn" },
+	{ "tyv", "Cyrl" },
+	{ "tzm", "Latn Tfng" },
+	{ "udm", "Cyrl Latn" },
+	{ "ug", "Arab Cyrl Latn" },
+	{ "uga", "Ugar" },
+	{ "uk", "Cyrl" },
+	{ "umb", "Latn" },
+	{ "unm", "Latn" },
+	{ "ur", "Arab" },
+	{ "uz", "Latn Cyrl Arab" },
+	{ "vai", "Vaii Latn" },
+	{ "ve", "Latn" },
+	{ "vec", "Latn" },
+	{ "vep", "Latn" },
+	{ "vi", "Latn Hani" },
+	{ "vls", "Latn" },
+	{ "vmf", "Latn" },
+	{ "vo", "Latn" },
+	{ "vot", "Latn" },
+	{ "vro", "Latn" },
+	{ "vun", "Latn" },
+	{ "wa", "Latn" },
+	{ "wae", "Latn" },
+	{ "wal", "Ethi" },
+	{ "war", "Latn" },
+	{ "was", "Latn" },
+	{ "wbp", "Latn" },
+	{ "wo", "Latn Arab" },
+	{ "wuu", "Hans Hani" },
+	{ "xal", "Cyrl" },
+	{ "xh", "Latn" },
+	{ "xmf", "Geor" },
+	{ "xog", "Latn" },
+	{ "yao", "Latn" },
+	{ "yap", "Latn" },
+	{ "yav", "Latn" },
+	{ "ybb", "Latn" },
+	{ "yi", "Hebr" },
+	{ "yo", "Latn" },
+	{ "yrl", "Latn" },
+	{ "yue", "Hant Hans Hani" },
+	{ "yuw", "Latn" },
+	{ "za", "Latn Hans Hani" },
+	{ "zap", "Latn" },
+	{ "zbl", "Blis" },
+	{ "zea", "Latn" },
+	{ "zen", "Tfng" },
+	{ "zgh", "Tfng" },
+	{ "zh", "Hans Hant Hani Bopo Latn Phag" },
+	{ "zu", "Latn" },
+	{ "zun", "Latn" },
+	{ "zza", "Latn" },
+	{ nullptr, nullptr },
 };
 
 // Additional regional variants.
@@ -1194,3 +1815,70 @@ static const char *script_list[][2] = {
 	{ "Zanabazar Square", "Zanb" },
 	{ nullptr, nullptr }
 };
+
+// Plural rules.
+// Reference:
+// - https://cgit.git.savannah.gnu.org/cgit/gettext.git/tree/gettext-tools/src/plural-table.c
+// - https://github.com/unicode-org/cldr/blob/release-47/common/supplemental/plurals.xml
+static const char *plural_rules[][2] = {
+	{ "bm bo dz hnj id ig ii in ja jbo jv jw kde kea km ko lkt lo ms my nqo osa root sah ses sg su th to tpi vi wo yo yue zh", "nplurals=1; plural=0;" },
+	{ "af an asa ast az bal bem bez bg brx ca ce cgg chr ckb da de dv ee el en eo es et eu fi fo fur fy gl gsw ha haw he hu ia io it jgo ji jmc ka kaj kcg kk kkj kl ks ksb ku ky lb lg lij mas mgo ml mn mr nah nb nd ne nl nn nnh no nr ny nyn om or os pap ps pt rm rof rwk saq sc sd sdh seh sn so sq ss ssy st sv sw syr ta te teo tig tk tn tr ts ug ur uz ve vo vun wae xh xog yi", "nplurals=2; plural=(n != 1);" },
+	{ "ak bho csw ff fr guw hy kab ln mg nso pa pt_BR si ti wa", "nplurals=2; plural=(n > 1);" },
+	{ "is mk", "nplurals=2; plural=(n%10==1 && n%100!=11);" },
+	{ "tzm", "nplurals=2; plural=(n<=1 || (n>=11 && n<=99));" },
+	{ "am as bn doi fa gu hi kn pcm zu", "nplurals=2; plural=(n==0 || n==1);" },
+	{ "ceb fil tl", "nplurals=2; plural=(n==1 || n==2 || n==3 || (n%10!=4 && n%10!=6 && n%10!=9));" },
+	{ "prg", "nplurals=3; plural=(n%10==0 || (n%100>=11 && n%100<=19) ? 0 : n%10==1 && n%100!=11 ? 1 : 2);" },
+	{ "lv", "nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n != 0 ? 1 : 2);" },
+	{ "lt", "nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && (n%100<10 || n%100>=20) ? 1 : 2);" },
+	{ "be hr ru sr uk", "nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2);" },
+	{ "bs sh", "nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<12 || n%100>14) ? 1 : 2);" },
+	{ "lag", "nplurals=3; plural=(n==0 ? 0 : (n==0 || n==1) && n!=0 ? 1 : 2);" },
+	{ "blo ksh", "nplurals=3; plural=(n==0 ? 0 : n==1 ? 1 : 2);" },
+	{ "shi", "nplurals=3; plural=(n==0 || n==1 ? 0 : n>=2 && n<=10 ? 1 : 2);" },
+	{ "pl", "nplurals=3; plural=(n==1 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2);" },
+	{ "mo", "nplurals=3; plural=(n==1 ? 0 : n==0 || (n!=1 && n%100>=1 && n%100<=19) ? 1 : 2);" },
+	{ "iu iw naq sat se sma smi smj smn sms", "nplurals=3; plural=(n==1 ? 0 : n==2 ? 1 : 2);" },
+	{ "cs sk", "nplurals=3; plural=(n==1) ? 0 : (n>=2 && n<=4) ? 1 : 2;" },
+	{ "ro", "nplurals=3; plural=n==1 ? 0 : (n==0 || (n%100 > 0 && n%100 < 20)) ? 1 : 2;" },
+	{ "ga", "nplurals=3; plural=n==1 ? 0 : n==2 ? 1 : 2;" },
+	{ "sl", "nplurals=4; plural=(n%100==1 ? 0 : n%100==2 ? 1 : n%100==3 || n%100==4 ? 2 : 3);" },
+	{ "dsb hsb", "nplurals=4; plural=(n%100==1 ? 0 : n%100==2 ? 1 : n%100>=3 && n%100<=4 ? 2 : 3);" },
+	{ "gv", "nplurals=4; plural=(n%10==1 ? 0 : n%10==2 ? 1 : n%100==0 || n%100==20 || n%100==40 || n%100==60 || n%100==80 ? 2 : 3);" },
+	{ "gd", "nplurals=4; plural=(n==1 || n==11 ? 0 : n==2 || n==12 ? 1 : (n>=3 && n<=10) || (n>=13 && n<=19) ? 2 : 3);" },
+	{ "br", "nplurals=5; plural=(n%10==1 && n%100!=11 && n%100!=71 && n%100!=91 ? 0 : n%10==2 && n%100!=12 && n%100!=72 && n%100!=92 ? 1 : ((n%10>=3 && n%10<=4) || n%10==9) && (n%100<10 || n%100>19) && (n%100<70 || n%100>79) && (n%100<90 || n%100>99) ? 2 : n!=0 && n%1000000==0 ? 3 : 4);" },
+	{ "mt", "nplurals=5; plural=(n==1 ? 0 : n==2 ? 1 : n==0 || (n%100>=3 && n%100<=10) ? 2 : n%100>=11 && n%100<=19 ? 3 : 4);" },
+	{ "kw", "nplurals=6; plural=(n==0 ? 0 : n==1 ? 1 : n%100==2 || n%100==22 || n%100==42 || n%100==62 || n%100==82 || (n%1000==0 && ((n%100000>=1000 && n%100000<=20000) || n%100000==40000 || n%100000==60000 || n%100000==80000)) || (n!=0 && n%1000000==100000) ? 2 : n%100==3 || n%100==23 || n%100==43 || n%100==63 || n%100==83 ? 3 : n!=1 && (n%100==1 || n%100==21 || n%100==41 || n%100==61 || n%100==81) ? 4 : 5);" },
+	{ "ar ars", "nplurals=6; plural=(n==0 ? 0 : n==1 ? 1 : n==2 ? 2 : n%100>=3 && n%100<=10 ? 3 : n%100>=11 && n%100<=99 ? 4 : 5);" },
+	{ "cy", "nplurals=6; plural=(n==0 ? 0 : n==1 ? 1 : n==2 ? 2 : n==3 ? 3 : n==6 ? 4 : 5);" },
+	{ nullptr, nullptr }
+};
+
+static const struct NumSystemData {
+	const char *locales;
+	const char32_t *digits;
+	const char32_t *percent_sign;
+	const char32_t *exp_l;
+	const char32_t *exp_u;
+} num_system_data[] = {
+	// Eastern Arabic numerals.
+	{ "ar ar_AE ar_BH ar_DJ ar_EG ar_ER ar_IL ar_IQ ar_JO ar_KM ar_KW ar_LB ar_MR ar_OM ar_PS ar_QA ar_SA ar_SD ar_SO ar_SS ar_SY ar_TD ar_YE ckb ckb_IQ ckb_IR sd sd_PK sd_Arab sd_Arab_PK", U"٠١٢٣٤٥٦٧٨٩٫", U"٪", U"اس", U"اس" },
+	// Persian and Urdu numerals.
+	{ "fa fa_AF fa_IR ks ks_IN ks_Arab ks_Arab_IN lrc lrc_IQ lrc_IR mzn mzn_IR pa_PK pa_Arab pa_Arab_PK ps ps_AF ps_PK ur_IN uz_AF uz_Arab uz_Arab_AF", U"۰۱۲۳۴۵۶۷۸۹٫", U"٪", U"اس", U"اس" },
+	// Bengali numerals.
+	{ "as as_IN bn bn_BD bn_IN mni mni_IN mni_Beng mni_Beng_IN", U"০১২৩৪৫৬৭৮৯.", U"%", U"e", U"E" },
+	// Devanagari numerals.
+	{ "mr mr_IN ne ne_IN ne_NP sa sa_IN", U"०१२३४५६७८९.", U"%", U"e", U"E" },
+	// Dzongkha numerals.
+	{ "dz dz_BT", U"༠༡༢༣༤༥༦༧༨༩.", U"%", U"e", U"E" },
+	// Santali numerals.
+	{ "sat sat_IN sat_Olck sat_Olck_IN", U"᱐᱑᱒᱓᱔᱕᱖᱗᱘᱙.", U"%", U"e", U"E" },
+	// Burmese numerals.
+	{ "my my_MM", U"၀၁၂၃၄၅၆၇၈၉.", U"%", U"e", U"E" },
+	// Chakma numerals.
+	{ "ccp ccp_BD ccp_IN", U"𑄶𑄷𑄸𑄹𑄺𑄻𑄼𑄽𑄾𑄿.", U"%", U"e", U"E" },
+	// Adlam numerals.
+	{ "ff ff_Adlm_BF ff_Adlm_CM ff_Adlm_GH ff_Adlm_GM ff_Adlm_GN ff_Adlm_GW ff_Adlm_LR ff_Adlm_MR ff_Adlm_NE ff_Adlm_NG ff_Adlm_SL ff_Adlm_SN", U"𞥐𞥑𞥒𞥓𞥔𞥕𞥖𞥗𞥘𞥙.", U"%", U"𞤉", U"𞤉" },
+	// End sentinel.
+	{ nullptr, nullptr, nullptr, nullptr, nullptr }
+};
