@@ -30,8 +30,10 @@
 
 #pragma once
 
+#include "core/object/object.h"
 #include "core/templates/simple_type.h"
 #include "core/typedefs.h"
+#include "core/variant/variant.h"
 
 #include <type_traits>
 
@@ -50,6 +52,7 @@ enum Metadata {
 	METADATA_REAL_IS_DOUBLE,
 	METADATA_INT_IS_CHAR16,
 	METADATA_INT_IS_CHAR32,
+	METADATA_OBJECT_IS_REQUIRED,
 };
 }
 
@@ -180,6 +183,44 @@ struct GetTypeInfo<T *, std::enable_if_t<std::is_base_of_v<Object, T>>> {
 	}
 };
 
+template <class T>
+class RequiredParam;
+
+template <class T>
+class RequiredResult;
+
+template <typename T>
+struct GetTypeInfo<RequiredParam<T>, std::enable_if_t<std::is_base_of_v<Object, T>>> {
+	static const Variant::Type VARIANT_TYPE = Variant::OBJECT;
+	static const GodotTypeInfo::Metadata METADATA = GodotTypeInfo::METADATA_OBJECT_IS_REQUIRED;
+
+	template <typename U = T, std::enable_if_t<std::is_base_of_v<RefCounted, U>, int> = 0>
+	static inline PropertyInfo get_class_info() {
+		return PropertyInfo(Variant::OBJECT, String(), PROPERTY_HINT_RESOURCE_TYPE, T::get_class_static());
+	}
+
+	template <typename U = T, std::enable_if_t<!std::is_base_of_v<RefCounted, U>, int> = 0>
+	static inline PropertyInfo get_class_info() {
+		return PropertyInfo(StringName(T::get_class_static()));
+	}
+};
+
+template <typename T>
+struct GetTypeInfo<RequiredResult<T>, std::enable_if_t<std::is_base_of_v<Object, T>>> {
+	static const Variant::Type VARIANT_TYPE = Variant::OBJECT;
+	static const GodotTypeInfo::Metadata METADATA = GodotTypeInfo::METADATA_OBJECT_IS_REQUIRED;
+
+	template <typename U = T, std::enable_if_t<std::is_base_of_v<RefCounted, U>, int> = 0>
+	static inline PropertyInfo get_class_info() {
+		return PropertyInfo(Variant::OBJECT, String(), PROPERTY_HINT_RESOURCE_TYPE, T::get_class_static());
+	}
+
+	template <typename U = T, std::enable_if_t<!std::is_base_of_v<RefCounted, U>, int> = 0>
+	static inline PropertyInfo get_class_info() {
+		return PropertyInfo(StringName(T::get_class_static()));
+	}
+};
+
 namespace GodotTypeInfo {
 namespace Internal {
 inline String enum_qualified_name_to_class_info_name(const String &p_qualified_name) {
@@ -205,7 +246,7 @@ inline String enum_qualified_name_to_class_info_name(const String &p_qualified_n
 	};
 
 template <typename T>
-inline StringName __constant_get_enum_name(T param, const String &p_constant) {
+inline StringName __constant_get_enum_name(T param) {
 	return GetTypeInfo<T>::get_class_info().class_name;
 }
 
@@ -230,7 +271,7 @@ inline StringName __constant_get_enum_name(T param, const String &p_constant) {
 	};
 
 template <typename T>
-inline StringName __constant_get_bitfield_name(T param, const String &p_constant) {
+inline StringName __constant_get_bitfield_name(T param) {
 	return GetTypeInfo<BitField<T>>::get_class_info().class_name;
 }
 #define CLASS_INFO(m_type) (GetTypeInfo<m_type *>::get_class_info())
@@ -244,3 +285,57 @@ struct ZeroInitializer {
 		}
 	}
 };
+
+namespace GodotTypeInfo {
+namespace Internal {
+
+template <typename T>
+Variant::Type get_variant_type() {
+	if constexpr (std::is_base_of_v<Object, T>) {
+		return Variant::Type::OBJECT;
+	} else {
+		return GetTypeInfo<T>::VARIANT_TYPE;
+	}
+}
+
+template <typename T>
+const String get_object_class_name_or_empty() {
+	if constexpr (std::is_base_of_v<Object, T>) {
+		return T::get_class_static();
+	} else {
+		return "";
+	}
+}
+
+template <typename T>
+const String get_variant_type_identifier() {
+	if constexpr (std::is_base_of_v<Object, T>) {
+		return T::get_class_static();
+	} else if constexpr (std::is_same_v<Variant, T>) {
+		return "Variant";
+	} else {
+		return Variant::get_type_name(GetTypeInfo<T>::VARIANT_TYPE);
+	}
+}
+
+} //namespace Internal
+} //namespace GodotTypeInfo
+
+template <typename T>
+struct GetTypeInfo<TypedArray<T>> {
+	static const Variant::Type VARIANT_TYPE = Variant::ARRAY;
+	static const GodotTypeInfo::Metadata METADATA = GodotTypeInfo::METADATA_NONE;
+	static inline PropertyInfo get_class_info() {
+		return PropertyInfo(Variant::ARRAY, String(), PROPERTY_HINT_ARRAY_TYPE, GodotTypeInfo::Internal::get_variant_type_identifier<T>());
+	}
+};
+
+template <typename K, typename V>
+struct GetTypeInfo<TypedDictionary<K, V>> {
+	static const Variant::Type VARIANT_TYPE = Variant::DICTIONARY;
+	static const GodotTypeInfo::Metadata METADATA = GodotTypeInfo::METADATA_NONE;
+	static inline PropertyInfo get_class_info() {
+		return PropertyInfo(Variant::DICTIONARY, String(), PROPERTY_HINT_DICTIONARY_TYPE,
+				vformat("%s;%s", GodotTypeInfo::Internal::get_variant_type_identifier<K>(), GodotTypeInfo::Internal::get_variant_type_identifier<V>()));
+	}
+};
