@@ -2473,9 +2473,6 @@ AnimationPlayerEditorPlugin::AnimationPlayerEditorPlugin() {
 }
 
 AnimationPlayerEditorPlugin::~AnimationPlayerEditorPlugin() {
-	if (dummy_player) {
-		memdelete(dummy_player);
-	}
 }
 
 // AnimationTrackKeyEditEditorPlugin
