@@ -33,7 +33,7 @@
 #include "core/io/file_access.h"
 #include "core/io/image.h"
 #include "core/io/resource_importer.h"
-#include "servers/rendering_server.h"
+#include "servers/rendering/rendering_server.h"
 
 class CompressedTexture2D;
 
