@@ -47,6 +47,7 @@ class PluginConfigDialog : public ConfirmationDialog {
 		MSG_ID_SUBFOLDER,
 		MSG_ID_SCRIPT,
 		MSG_ID_ACTIVE,
+		MSG_ID_ENABLE_WARNINGS,
 	};
 
 	LineEdit *name_edit = nullptr;
@@ -56,7 +57,6 @@ class PluginConfigDialog : public ConfirmationDialog {
 	LineEdit *version_edit = nullptr;
 	OptionButton *script_option_edit = nullptr;
 	LineEdit *script_edit = nullptr;
-	CheckBox *active_edit = nullptr;
 
 	LocalVector<Control *> plugin_edit_hidden_controls;
 
