@@ -64,6 +64,12 @@ private:
 		INSTALL_PATH,
 	};
 
+	enum InvalidStateFlag {
+		INVALID_STATE_FLAG_NONE = 0,
+		INVALID_STATE_FLAG_PATH_INPUT = 1 << 0,
+		INVALID_STATE_FLAG_RENDERER_SELECT = 1 << 1,
+	};
+
 	Mode mode = MODE_NEW;
 	bool is_folder_empty = true;
 	ConfirmationDialog *nonempty_confirmation = nullptr;
@@ -80,6 +86,7 @@ private:
 	HBoxContainer *default_files_container = nullptr;
 	Ref<ButtonGroup> renderer_button_group;
 	bool rendering_device_supported = false;
+	bool rendering_device_checked = false;
 	Label *rd_not_supported = nullptr;
 
 	Label *msg = nullptr;
@@ -103,7 +110,10 @@ private:
 	String original_project_path;
 	bool duplicate_can_edit = false;
 
+	BitField<InvalidStateFlag> invalid_state_flags = INVALID_STATE_FLAG_NONE;
+
 	void _set_message(const String &p_msg, MessageType p_type, InputType input_type = PROJECT_PATH);
+	void _update_ok_button();
 	void _validate_path();
 
 	// Project path for MODE_NEW and MODE_INSTALL. Install path for MODE_IMPORT.
@@ -151,7 +161,7 @@ public:
 	void set_duplicate_can_edit(bool p_duplicate_can_edit);
 
 	void ask_for_path_and_show();
-	void show_dialog(bool p_reset_name = true);
+	void show_dialog(bool p_reset_name = true, bool p_is_confirmed = true);
 
 	ProjectDialog();
 };
