@@ -492,7 +492,7 @@ void ScriptCreateDialog::_file_selected(const String &p_file) {
 }
 
 void ScriptCreateDialog::_create() {
-	parent_name->set_text(select_class->get_selected_type());
+	parent_name->set_text(select_class->get_selected_type_name());
 	_parent_name_changed(parent_name->get_text());
 }
 
