@@ -1,3 +1,4 @@
+using System.Threading.Tasks;
 using Xunit;
 
 namespace Godot.SourceGenerators.Tests;
@@ -5,7 +6,7 @@ namespace Godot.SourceGenerators.Tests;
 public class KeywordClassAndNamespaceTest
 {
     [Fact]
-    public async void GenerateScriptMethodsTest()
+    public async Task GenerateScriptMethodsTest()
     {
         await CSharpSourceGeneratorVerifier<ScriptMethodsGenerator>.Verify(
             "KeywordClassNameAndNamespace.cs",
