@@ -44,7 +44,7 @@ void OpenXRSelectActionDialog::_notification(int p_what) {
 	}
 }
 
-void OpenXRSelectActionDialog::_on_select_action(const String p_action) {
+void OpenXRSelectActionDialog::_on_select_action(const String &p_action) {
 	if (selected_action != "") {
 		NodePath button_path = action_buttons[selected_action];
 		Button *button = Object::cast_to<Button>(get_node(button_path));
@@ -120,7 +120,7 @@ void OpenXRSelectActionDialog::ok_pressed() {
 	hide();
 }
 
-OpenXRSelectActionDialog::OpenXRSelectActionDialog(Ref<OpenXRActionMap> p_action_map) {
+OpenXRSelectActionDialog::OpenXRSelectActionDialog(const Ref<OpenXRActionMap> &p_action_map) {
 	action_map = p_action_map;
 
 	set_title(TTR("Select an action"));
