@@ -73,12 +73,12 @@ public:
 	virtual GatheringState get_gathering_state() const = 0;
 	virtual SignalingState get_signaling_state() const = 0;
 
-	virtual Error initialize(Dictionary p_config = Dictionary()) = 0;
-	virtual Ref<WebRTCDataChannel> create_data_channel(String p_label, Dictionary p_options = Dictionary()) = 0;
+	virtual Error initialize(const Dictionary &p_config = Dictionary()) = 0;
+	virtual Ref<WebRTCDataChannel> create_data_channel(const String &p_label, const Dictionary &p_options = Dictionary()) = 0;
 	virtual Error create_offer() = 0;
-	virtual Error set_remote_description(String type, String sdp) = 0;
-	virtual Error set_local_description(String type, String sdp) = 0;
-	virtual Error add_ice_candidate(String sdpMidName, int sdpMlineIndexName, String sdpName) = 0;
+	virtual Error set_remote_description(const String &p_type, const String &p_sdp) = 0;
+	virtual Error set_local_description(const String &p_type, const String &p_sdp) = 0;
+	virtual Error add_ice_candidate(const String &p_sdp_mid_name, int p_sdp_mline_index_name, const String &p_sdp_name) = 0;
 	virtual Error poll() = 0;
 	virtual void close() = 0;
 
