@@ -31,7 +31,7 @@
 #include "character_body_2d.h"
 
 #ifndef DISABLE_DEPRECATED
-#include "servers/extensions/physics_server_2d_extension.h"
+#include "servers/physics_2d/physics_server_2d_extension.h"
 #endif
 
 // So, if you pass 45 as limit, avoid numerical precision errors when angle is 45.
