@@ -558,6 +558,7 @@ void LookAtModifier3D::_process_modification(double p_delta) {
 			forward_vector = Vector3(0, 0, 0); // The zero-vector to be used for checking in the line immediately below to avoid animation glitch.
 		} else {
 			destination = look_at_with_axes(bone_rest).basis.get_rotation_quaternion();
+			forward_vector = bone_rest.basis.xform_inv(forward_vector_nrm); // Forward vector in the "current" bone rest.
 		}
 	}
 
@@ -573,18 +574,17 @@ void LookAtModifier3D::_process_modification(double p_delta) {
 		init_transition();
 	} else if (is_flippable && std::signbit(prev_forward_vector[secondary_rotation_axis]) != std::signbit(forward_vector[secondary_rotation_axis])) {
 		// Flipping by angle_limitation can be detected by sign of secondary rotation axes during forward_vector is rotated more than 90 degree from forward_axis (means dot production is negative).
-		Vector3 prev_forward_vector_nrm = forward_vector.normalized();
 		Vector3 rest_forward_vector = get_vector_from_bone_axis(forward_axis);
 		if (symmetry_limitation) {
 			if ((is_not_max_influence || !Math::is_equal_approx(primary_limit_angle, (float)Math::TAU)) &&
-					prev_forward_vector_nrm.dot(rest_forward_vector) < 0 &&
-					forward_vector_nrm.dot(rest_forward_vector) < 0) {
+					prev_forward_vector.dot(rest_forward_vector) < 0 &&
+					forward_vector.dot(rest_forward_vector) < 0) {
 				init_transition();
 			}
 		} else {
 			if ((is_not_max_influence || !Math::is_equal_approx(primary_positive_limit_angle + primary_negative_limit_angle, (float)Math::TAU)) &&
-					prev_forward_vector_nrm.dot(rest_forward_vector) < 0 &&
-					forward_vector_nrm.dot(rest_forward_vector) < 0) {
+					prev_forward_vector.dot(rest_forward_vector) < 0 &&
+					forward_vector.dot(rest_forward_vector) < 0) {
 				init_transition();
 			}
 		}
@@ -595,7 +595,7 @@ void LookAtModifier3D::_process_modification(double p_delta) {
 		remaining = MAX(0, remaining - time_step * p_delta);
 		if (is_flippable) {
 			// Interpolate through the rest same as AnimationTree blending for preventing to penetrate the bone into the body.
-			Quaternion rest = skeleton->get_bone_rest(bone).basis.get_rotation_quaternion();
+			Quaternion rest = bone_rest.basis.get_rotation_quaternion();
 			float weight = Tween::run_equation(transition_type, ease_type, 1 - remaining, 0.0, 1.0, 1.0);
 			destination = Animation::interpolate_via_rest(Animation::interpolate_via_rest(rest, from_q, 1 - weight, rest), destination, weight, rest);
 		} else {
