@@ -129,9 +129,6 @@ void Occluder3D::_notification(int p_what) {
 	}
 }
 
-void Occluder3D::_bind_methods() {
-}
-
 Occluder3D::Occluder3D() {
 	occluder = RS::get_singleton()->occluder_create();
 }
@@ -139,7 +136,7 @@ Occluder3D::Occluder3D() {
 Occluder3D::~Occluder3D() {
 	if (occluder.is_valid()) {
 		ERR_FAIL_NULL(RenderingServer::get_singleton());
-		RS::get_singleton()->free(occluder);
+		RS::get_singleton()->free_rid(occluder);
 	}
 }
 
@@ -324,13 +321,13 @@ void SphereOccluder3D::_update_arrays(PackedVector3Array &r_vertices, PackedInt3
 	int point = 0;
 	for (int j = 0; j <= (RINGS + 1); j++) {
 		float v = j / float(RINGS + 1);
-		float w = Math::sin(Math_PI * v);
-		float y = Math::cos(Math_PI * v);
+		float w = Math::sin(Math::PI * v);
+		float y = Math::cos(Math::PI * v);
 		for (int i = 0; i <= RADIAL_SEGMENTS; i++) {
 			float u = i / float(RADIAL_SEGMENTS);
 
-			float x = Math::cos(u * Math_TAU);
-			float z = Math::sin(u * Math_TAU);
+			float x = Math::cos(u * Math::TAU);
+			float z = Math::sin(u * Math::TAU);
 			vertex_ptr[vertex_i++] = Vector3(x * w, y, z * w) * radius;
 
 			if (i > 0 && j > 0) {
@@ -524,7 +521,7 @@ void OccluderInstance3D::_bake_surface(const Transform3D &p_transform, Array p_s
 	PackedVector3Array vertices = p_surface_arrays[Mesh::ARRAY_VERTEX];
 	PackedInt32Array indices = p_surface_arrays[Mesh::ARRAY_INDEX];
 
-	if (vertices.size() == 0 || indices.size() == 0) {
+	if (vertices.is_empty() || indices.is_empty()) {
 		return;
 	}
 
@@ -694,9 +691,9 @@ OccluderInstance3D::BakeError OccluderInstance3D::bake_scene(Node *p_from_node,
 }
 
 PackedStringArray OccluderInstance3D::get_configuration_warnings() const {
-	PackedStringArray warnings = Node::get_configuration_warnings();
+	PackedStringArray warnings = VisualInstance3D::get_configuration_warnings();
 
-	if (!bool(GLOBAL_GET("rendering/occlusion_culling/use_occlusion_culling"))) {
+	if (!bool(GLOBAL_GET_CACHED(bool, "rendering/occlusion_culling/use_occlusion_culling"))) {
 		warnings.push_back(RTR("Occlusion culling is disabled in the Project Settings, which means occlusion culling won't be performed in the root viewport.\nTo resolve this, open the Project Settings and enable Rendering > Occlusion Culling > Use Occlusion Culling."));
 	}
 
