@@ -34,6 +34,8 @@
 #include "scene/property_list_helper.h"
 #include "scene/resources/text_line.h"
 
+class Timer;
+
 class TabBar : public Control {
 	GDCLASS(TabBar, Control);
 
@@ -52,11 +54,22 @@ public:
 		CLOSE_BUTTON_MAX
 	};
 
+	enum DrawMode {
+		DRAW_NORMAL,
+		DRAW_PRESSED,
+		DRAW_HOVER,
+		DRAW_DISABLED,
+		DRAW_MAX,
+	};
+
 private:
 	struct Tab {
 		mutable RID accessibility_item_element;
 		mutable bool accessibility_item_dirty = true;
 
+		// Corresponds to color overrides for the DrawMode enum
+		Color font_color_overrides[DrawMode::DRAW_MAX] = { Color(0, 0, 0, 0), Color(0, 0, 0, 0), Color(0, 0, 0, 0), Color(0, 0, 0, 0) };
+
 		String text;
 		String tooltip;
 
@@ -106,6 +119,7 @@ private:
 
 	bool select_with_rmb = false;
 	bool deselect_enabled = false;
+	bool switch_on_release = false;
 
 	int cb_hover = -1;
 	bool cb_pressing = false;
@@ -119,6 +133,7 @@ private:
 	bool dragging_valid_tab = false;
 	bool scroll_to_selected = true;
 	int tabs_rearrange_group = -1;
+	bool switch_on_drag_hover = true;
 
 	static const int CURRENT_TAB_UNINITIALIZED = -2;
 	bool initialized = false;
@@ -132,6 +147,7 @@ private:
 		int h_separation = 0;
 		int tab_separation = 0;
 		int icon_max_width = 0;
+		int hover_switch_wait_msec = 500;
 
 		Ref<StyleBox> tab_unselected_style;
 		Ref<StyleBox> tab_hovered_style;
@@ -156,11 +172,18 @@ private:
 		Color font_disabled_color;
 		Color font_outline_color;
 
+		Color icon_selected_color;
+		Color icon_hovered_color;
+		Color icon_unselected_color;
+		Color icon_disabled_color;
+
 		Ref<Texture2D> close_icon;
 		Ref<StyleBox> button_pressed_style;
 		Ref<StyleBox> button_hl_style;
 	} theme_cache;
 
+	Timer *hover_switch_delay = nullptr;
+
 	int get_tab_width(int p_idx) const;
 	Size2 _get_tab_icon_size(int p_idx) const;
 	void _ensure_no_over_offset();
@@ -168,11 +191,12 @@ private:
 
 	void _update_hover();
 	void _update_cache(bool p_update_hover = true);
+	void _hover_switch_timeout();
 
 	void _on_mouse_exited();
 
 	void _shape(int p_tab);
-	void _draw_tab(Ref<StyleBox> &p_tab_style, Color &p_font_color, int p_index, float p_x, bool p_focus);
+	void _draw_tab(Ref<StyleBox> &p_tab_style, const Color &p_font_color, const Color &p_icon_color, int p_index, float p_x, bool p_focus);
 
 	void _accessibility_action_scroll_into_view(const Variant &p_data, int p_index);
 	void _accessibility_action_focus(const Variant &p_data, int p_index);
@@ -202,6 +226,7 @@ public:
 	bool _handle_can_drop_data(const String &p_type, const Point2 &p_point, const Variant &p_data) const;
 	void _handle_drop_data(const String &p_type, const Point2 &p_point, const Variant &p_data, const Callable &p_move_tab_callback, const Callable &p_move_tab_from_other_callback);
 	void _draw_tab_drop(RID p_canvas_item);
+	void _handle_switch_on_hover(const Variant &p_data) const;
 
 	void add_tab(const String &p_str = "", const Ref<Texture2D> &p_icon = Ref<Texture2D>());
 
@@ -223,6 +248,10 @@ public:
 	void set_tab_icon_max_width(int p_tab, int p_width);
 	int get_tab_icon_max_width(int p_tab) const;
 
+	void set_font_color_override_all(int p_tab, const Color &p_color);
+	void set_font_color_override(int p_tab, DrawMode p_draw_mode, const Color &p_color);
+	Color get_font_color_override(int p_tab, DrawMode p_draw_mode) const;
+
 	void set_tab_disabled(int p_tab, bool p_disabled);
 	bool is_tab_disabled(int p_tab) const;
 
@@ -287,6 +316,9 @@ public:
 	void set_scroll_to_selected(bool p_enabled);
 	bool get_scroll_to_selected() const;
 
+	void set_switch_on_drag_hover(bool p_enabled);
+	bool get_switch_on_drag_hover() const;
+
 	void set_select_with_rmb(bool p_enabled);
 	bool get_select_with_rmb() const;
 
@@ -298,6 +330,8 @@ public:
 	void set_max_tab_width(int p_width);
 	int get_max_tab_width() const;
 
+	void set_switch_on_release(bool p_switch) { switch_on_release = p_switch; }
+
 	Rect2 get_tab_rect(int p_tab) const;
 	Size2 get_minimum_size() const override;
 
