@@ -35,7 +35,7 @@
 #include "scene/resources/camera_attributes.h"
 #include "scene/resources/environment.h"
 #ifndef NAVIGATION_3D_DISABLED
-#include "servers/navigation_server_3d.h"
+#include "servers/navigation_3d/navigation_server_3d.h"
 #endif // NAVIGATION_3D_DISABLED
 
 void World3D::_register_camera(Camera3D *p_camera) {
@@ -194,17 +194,17 @@ World3D::~World3D() {
 	ERR_FAIL_NULL(NavigationServer3D::get_singleton());
 #endif // NAVIGATION_3D_DISABLED
 
-	RenderingServer::get_singleton()->free(scenario);
+	RenderingServer::get_singleton()->free_rid(scenario);
 
 #ifndef PHYSICS_3D_DISABLED
 	if (space.is_valid()) {
-		PhysicsServer3D::get_singleton()->free(space);
+		PhysicsServer3D::get_singleton()->free_rid(space);
 	}
 #endif // PHYSICS_3D_DISABLED
 
 #ifndef NAVIGATION_3D_DISABLED
 	if (navigation_map.is_valid()) {
-		NavigationServer3D::get_singleton()->free(navigation_map);
+		NavigationServer3D::get_singleton()->free_rid(navigation_map);
 	}
 #endif // NAVIGATION_3D_DISABLED
 }
