@@ -28,21 +28,53 @@
 /* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
 /**************************************************************************/
 
-#ifndef RENDERING_DEVICE_COMMONS_H
-#define RENDERING_DEVICE_COMMONS_H
+#pragma once
 
 #include "core/object/object.h"
 #include "core/variant/type_info.h"
 
-#define STEPIFY(m_number, m_alignment) ((((m_number) + ((m_alignment)-1)) / (m_alignment)) * (m_alignment))
+#define STEPIFY(m_number, m_alignment) ((((m_number) + ((m_alignment) - 1)) / (m_alignment)) * (m_alignment))
+
+// This may one day be used in Godot for interoperability between C arrays, Vector and LocalVector.
+// (See https://github.com/godotengine/godot-proposals/issues/5144.)
+template <typename T>
+class VectorView {
+	const T *_ptr = nullptr;
+	const uint32_t _size = 0;
+
+public:
+	const T &operator[](uint32_t p_index) {
+		DEV_ASSERT(p_index < _size);
+		return _ptr[p_index];
+	}
+
+	_ALWAYS_INLINE_ const T *ptr() const { return _ptr; }
+	_ALWAYS_INLINE_ uint32_t size() const { return _size; }
+
+	VectorView() = default;
+	VectorView(const T &p_ptr) :
+			// With this one you can pass a single element very conveniently!
+			_ptr(&p_ptr),
+			_size(1) {}
+	VectorView(const T *p_ptr, uint32_t p_size) :
+			_ptr(p_ptr), _size(p_size) {}
+	VectorView(const Vector<T> &p_lv) :
+			_ptr(p_lv.ptr()), _size(p_lv.size()) {}
+	VectorView(const LocalVector<T> &p_lv) :
+			_ptr(p_lv.ptr()), _size(p_lv.size()) {}
+};
 
 class RenderingDeviceCommons : public Object {
+	GDSOFTCLASS(RenderingDeviceCommons, Object);
+
 	////////////////////////////////////////////
 	// PUBLIC STUFF
 	// Exposed by RenderingDevice, and shared
 	// with RenderingDeviceDriver.
 	////////////////////////////////////////////
 public:
+	static const bool command_pool_reset_enabled = true;
+
 	/*****************/
 	/**** GENERIC ****/
 	/*****************/
@@ -268,9 +300,61 @@ public:
 		DATA_FORMAT_G16_B16_R16_3PLANE_422_UNORM,
 		DATA_FORMAT_G16_B16R16_2PLANE_422_UNORM,
 		DATA_FORMAT_G16_B16_R16_3PLANE_444_UNORM,
+		DATA_FORMAT_ASTC_4x4_SFLOAT_BLOCK, // HDR variant.
+		DATA_FORMAT_ASTC_5x4_SFLOAT_BLOCK, // HDR variant.
+		DATA_FORMAT_ASTC_5x5_SFLOAT_BLOCK, // HDR variant.
+		DATA_FORMAT_ASTC_6x5_SFLOAT_BLOCK, // HDR variant.
+		DATA_FORMAT_ASTC_6x6_SFLOAT_BLOCK, // HDR variant.
+		DATA_FORMAT_ASTC_8x5_SFLOAT_BLOCK, // HDR variant.
+		DATA_FORMAT_ASTC_8x6_SFLOAT_BLOCK, // HDR variant.
+		DATA_FORMAT_ASTC_8x8_SFLOAT_BLOCK, // HDR variant.
+		DATA_FORMAT_ASTC_10x5_SFLOAT_BLOCK, // HDR variant.
+		DATA_FORMAT_ASTC_10x6_SFLOAT_BLOCK, // HDR variant.
+		DATA_FORMAT_ASTC_10x8_SFLOAT_BLOCK, // HDR variant.
+		DATA_FORMAT_ASTC_10x10_SFLOAT_BLOCK, // HDR variant.
+		DATA_FORMAT_ASTC_12x10_SFLOAT_BLOCK, // HDR variant.
+		DATA_FORMAT_ASTC_12x12_SFLOAT_BLOCK, // HDR variant.
 		DATA_FORMAT_MAX,
 	};
 
+	// Breadcrumb markers are useful for debugging GPU crashes (i.e. DEVICE_LOST). Internally
+	// they're just an uint32_t to "tag" a GPU command. These are only used for debugging and do not
+	// (or at least shouldn't) alter the execution behavior in any way.
+	//
+	// When a GPU crashes and Godot was built in dev or debug mode; Godot will dump what commands
+	// were being executed and what tag they were marked with.
+	// This makes narrowing down the cause of a crash easier. Note that a GPU can be executing
+	// multiple commands at the same time. It is also useful to identify data hazards.
+	//
+	// For example if each LIGHTMAPPER_PASS must be executed in sequential order, but dumps
+	// indicated that pass (LIGHTMAPPER_PASS | 5) was being executed at the same time as
+	// (LIGHTMAPPER_PASS | 4), that would indicate there is a missing barrier or a render graph bug.
+	//
+	// The enums are bitshifted by 16 bits so it's possible to add user data via bitwise operations.
+	// Using this enum is not mandatory; but it is recommended so that all subsystems agree what each
+	// ID means when dumping info.
+	enum BreadcrumbMarker {
+		NONE = 0,
+		// Environment
+		REFLECTION_PROBES = 1u << 16u,
+		SKY_PASS = 2u << 16u,
+		// Light mapping
+		LIGHTMAPPER_PASS = 3u << 16u,
+		// Shadows
+		SHADOW_PASS_DIRECTIONAL = 4u << 16u,
+		SHADOW_PASS_CUBE = 5u << 16u,
+		// Geometry passes
+		OPAQUE_PASS = 6u << 16u,
+		ALPHA_PASS = 7u << 16u,
+		TRANSPARENT_PASS = 8u << 16u,
+		// Screen effects
+		POST_PROCESSING_PASS = 9u << 16u,
+		BLIT_PASS = 10u << 16u,
+		UI_PASS = 11u << 16u,
+		// Other
+		DEBUG_PASS = 12u << 16u,
+	};
+
 	enum CompareOperator {
 		COMPARE_OP_NEVER,
 		COMPARE_OP_LESS,
@@ -321,6 +405,24 @@ public:
 		TEXTURE_USAGE_CAN_COPY_TO_BIT = (1 << 8),
 		TEXTURE_USAGE_INPUT_ATTACHMENT_BIT = (1 << 9),
 		TEXTURE_USAGE_VRS_ATTACHMENT_BIT = (1 << 10),
+		// When set, the texture is not backed by actual memory. It only ever lives in the cache.
+		// This is particularly useful for:
+		//	1. Depth/stencil buffers that are not needed after producing the colour output.
+		//	2. MSAA surfaces that are immediately resolved (i.e. its raw content isn't needed).
+		//
+		// This flag heavily improves performance & saves memory on TBDR GPUs (e.g. mobile).
+		// On Desktop this flag won't save memory but it still instructs the render graph that data will
+		// be discarded aggressively which may still improve some performance.
+		//
+		// It is not valid to perform copies from/to this texture, since it doesn't occupy actual RAM.
+		// It is also not valid to sample from this texture except using subpasses or via read/write
+		// pixel shader extensions (e.g. VK_EXT_rasterization_order_attachment_access).
+		//
+		// Try to set this bit as much as possible. If you set it, validation doesn't complain
+		// and it works fine on mobile, then go ahead.
+		TEXTURE_USAGE_TRANSIENT_BIT = (1 << 11),
+		TEXTURE_USAGE_DEPTH_RESOLVE_ATTACHMENT_BIT = (1 << 12),
+		TEXTURE_USAGE_MAX_BIT = TEXTURE_USAGE_DEPTH_RESOLVE_ATTACHMENT_BIT,
 	};
 
 	struct TextureFormat {
@@ -335,6 +437,7 @@ public:
 		uint32_t usage_bits = 0;
 		Vector<DataFormat> shareable_formats;
 		bool is_resolve_buffer = false;
+		bool is_discardable = false;
 
 		bool operator==(const TextureFormat &b) const {
 			if (format != b.format) {
@@ -357,6 +460,10 @@ public:
 				return false;
 			} else if (shareable_formats != b.shareable_formats) {
 				return false;
+			} else if (is_resolve_buffer != b.is_resolve_buffer) {
+				return false;
+			} else if (is_discardable != b.is_discardable) {
+				return false;
 			} else {
 				return true;
 			}
@@ -443,6 +550,7 @@ public:
 	};
 
 	struct VertexAttribute {
+		uint32_t binding = UINT32_MAX; // Attribute buffer binding index. When set to UINT32_MAX, it uses the index of the attribute in the layout.
 		uint32_t location = 0; // Shader location.
 		uint32_t offset = 0;
 		DataFormat format = DATA_FORMAT_MAX;
@@ -450,6 +558,18 @@ public:
 		VertexFrequency frequency = VERTEX_FREQUENCY_VERTEX;
 	};
 
+	struct VertexAttributeBinding {
+		uint32_t stride = 0;
+		VertexFrequency frequency = VERTEX_FREQUENCY_VERTEX;
+
+		VertexAttributeBinding() = default;
+		VertexAttributeBinding(uint32_t p_stride, VertexFrequency p_frequency) :
+				stride(p_stride),
+				frequency(p_frequency) {}
+	};
+
+	typedef HashMap<uint32_t, VertexAttributeBinding> VertexAttributeBindingsMap;
+
 	/*********************/
 	/**** FRAMEBUFFER ****/
 	/*********************/
@@ -474,9 +594,34 @@ public:
 		SHADER_STAGE_COMPUTE_BIT = (1 << SHADER_STAGE_COMPUTE),
 	};
 
+	enum ShaderLanguage {
+		SHADER_LANGUAGE_GLSL,
+		SHADER_LANGUAGE_HLSL,
+	};
+
+	enum ShaderLanguageVersion {
+		SHADER_LANGUAGE_VULKAN_VERSION_1_0 = (1 << 22),
+		SHADER_LANGUAGE_VULKAN_VERSION_1_1 = (1 << 22) | (1 << 12),
+		SHADER_LANGUAGE_VULKAN_VERSION_1_2 = (1 << 22) | (2 << 12),
+		SHADER_LANGUAGE_VULKAN_VERSION_1_3 = (1 << 22) | (3 << 12),
+		SHADER_LANGUAGE_VULKAN_VERSION_1_4 = (1 << 22) | (4 << 12),
+		SHADER_LANGUAGE_OPENGL_VERSION_4_5_0 = 450,
+	};
+
+	enum ShaderSpirvVersion {
+		SHADER_SPIRV_VERSION_1_0 = (1 << 16),
+		SHADER_SPIRV_VERSION_1_1 = (1 << 16) | (1 << 8),
+		SHADER_SPIRV_VERSION_1_2 = (1 << 16) | (2 << 8),
+		SHADER_SPIRV_VERSION_1_3 = (1 << 16) | (3 << 8),
+		SHADER_SPIRV_VERSION_1_4 = (1 << 16) | (4 << 8),
+		SHADER_SPIRV_VERSION_1_5 = (1 << 16) | (5 << 8),
+		SHADER_SPIRV_VERSION_1_6 = (1 << 16) | (6 << 8),
+	};
+
 	struct ShaderStageSPIRVData {
 		ShaderStage shader_stage = SHADER_STAGE_MAX;
 		Vector<uint8_t> spirv;
+		Vector<uint64_t> dynamic_buffers;
 	};
 
 	/*********************/
@@ -485,6 +630,7 @@ public:
 
 	static const uint32_t MAX_UNIFORM_SETS = 16;
 
+	// Keep the enum values in sync with the `SHADER_UNIFORM_NAMES` values (file rendering_device.cpp).
 	enum UniformType {
 		UNIFORM_TYPE_SAMPLER, // For sampling only (sampler GLSL type).
 		UNIFORM_TYPE_SAMPLER_WITH_TEXTURE, // For sampling only, but includes a texture, (samplerXX GLSL type), first a sampler then a texture.
@@ -496,6 +642,8 @@ public:
 		UNIFORM_TYPE_UNIFORM_BUFFER, // Regular uniform buffer (or UBO).
 		UNIFORM_TYPE_STORAGE_BUFFER, // Storage buffer ("buffer" qualifier) like UBO, but supports storage, for compute mostly.
 		UNIFORM_TYPE_INPUT_ATTACHMENT, // Used for sub-pass read/write, for mobile mostly.
+		UNIFORM_TYPE_UNIFORM_BUFFER_DYNAMIC, // Same as UNIFORM but created with BUFFER_USAGE_DYNAMIC_PERSISTENT_BIT.
+		UNIFORM_TYPE_STORAGE_BUFFER_DYNAMIC, // Same as STORAGE but created with BUFFER_USAGE_DYNAMIC_PERSISTENT_BIT.
 		UNIFORM_TYPE_MAX
 	};
 
@@ -534,7 +682,7 @@ public:
 		RENDER_PRIMITIVE_TRIANGLES,
 		RENDER_PRIMITIVE_TRIANGLES_WITH_ADJACENCY,
 		RENDER_PRIMITIVE_TRIANGLE_STRIPS,
-		RENDER_PRIMITIVE_TRIANGLE_STRIPS_WITH_AJACENCY,
+		RENDER_PRIMITIVE_TRIANGLE_STRIPS_WITH_AJACENCY, // TODO: Fix typo in "ADJACENCY" (in 5.0).
 		RENDER_PRIMITIVE_TRIANGLE_STRIPS_WITH_RESTART_INDEX,
 		RENDER_PRIMITIVE_TESSELATION_PATCH,
 		RENDER_PRIMITIVE_MAX
@@ -808,18 +956,24 @@ public:
 		LIMIT_SUBGROUP_MAX_SIZE,
 		LIMIT_SUBGROUP_IN_SHADERS, // Set flags using SHADER_STAGE_VERTEX_BIT, SHADER_STAGE_FRAGMENT_BIT, etc.
 		LIMIT_SUBGROUP_OPERATIONS,
-		LIMIT_VRS_TEXEL_WIDTH,
-		LIMIT_VRS_TEXEL_HEIGHT,
-		LIMIT_VRS_MAX_FRAGMENT_WIDTH,
-		LIMIT_VRS_MAX_FRAGMENT_HEIGHT,
+		LIMIT_METALFX_TEMPORAL_SCALER_MIN_SCALE = 46,
+		LIMIT_METALFX_TEMPORAL_SCALER_MAX_SCALE,
+		LIMIT_MAX_SHADER_VARYINGS,
 	};
 
 	enum Features {
 		SUPPORTS_MULTIVIEW,
-		SUPPORTS_FSR_HALF_FLOAT,
+		SUPPORTS_HALF_FLOAT,
 		SUPPORTS_ATTACHMENT_VRS,
-		// If not supported, a fragment shader with only side effets (i.e., writes  to buffers, but doesn't output to attachments), may be optimized down to no-op by the GPU driver.
+		SUPPORTS_METALFX_SPATIAL,
+		SUPPORTS_METALFX_TEMPORAL,
+		// If not supported, a fragment shader with only side effects (i.e., writes  to buffers, but doesn't output to attachments), may be optimized down to no-op by the GPU driver.
 		SUPPORTS_FRAGMENT_SHADER_WITH_ONLY_SIDE_EFFECTS,
+		SUPPORTS_BUFFER_DEVICE_ADDRESS,
+		SUPPORTS_IMAGE_ATOMIC_32_BIT,
+		SUPPORTS_VULKAN_MEMORY_MODEL,
+		SUPPORTS_FRAMEBUFFER_DEPTH_RESOLVE,
+		SUPPORTS_POINT_SIZE,
 	};
 
 	enum SubgroupOperations {
@@ -853,12 +1007,12 @@ protected:
 
 	static const uint32_t TEXTURE_SAMPLES_COUNT[TEXTURE_SAMPLES_MAX];
 
-	static uint32_t get_image_format_pixel_size(DataFormat p_format);
 	static void get_compressed_image_format_block_dimensions(DataFormat p_format, uint32_t &r_w, uint32_t &r_h);
-	uint32_t get_compressed_image_format_block_byte_size(DataFormat p_format);
+	uint32_t get_compressed_image_format_block_byte_size(DataFormat p_format) const;
 	static uint32_t get_compressed_image_format_pixel_rshift(DataFormat p_format);
 	static uint32_t get_image_format_required_size(DataFormat p_format, uint32_t p_width, uint32_t p_height, uint32_t p_depth, uint32_t p_mipmaps, uint32_t *r_blockw = nullptr, uint32_t *r_blockh = nullptr, uint32_t *r_depth = nullptr);
 	static uint32_t get_image_required_mipmaps(uint32_t p_width, uint32_t p_height, uint32_t p_depth);
+	static bool format_has_depth(DataFormat p_format);
 	static bool format_has_stencil(DataFormat p_format);
 	static uint32_t format_get_plane_count(DataFormat p_format);
 
@@ -874,18 +1028,24 @@ protected:
 
 	static uint32_t get_format_vertex_size(DataFormat p_format);
 
+public:
+	/*****************/
+	/**** TEXTURE ****/
+	/*****************/
+
+	static uint32_t get_image_format_pixel_size(DataFormat p_format);
+
 	/****************/
 	/**** SHADER ****/
 	/****************/
 
 	static const char *SHADER_STAGE_NAMES[SHADER_STAGE_MAX];
 
-public:
 	struct ShaderUniform {
 		UniformType type = UniformType::UNIFORM_TYPE_MAX;
 		bool writable = false;
 		uint32_t binding = 0;
-		BitField<ShaderStage> stages;
+		BitField<ShaderStage> stages = {};
 		uint32_t length = 0; // Size of arrays (in total elements), or ubos (in bytes * total elements).
 
 		bool operator!=(const ShaderUniform &p_other) const {
@@ -913,28 +1073,24 @@ public:
 	};
 
 	struct ShaderSpecializationConstant : public PipelineSpecializationConstant {
-		BitField<ShaderStage> stages;
+		BitField<ShaderStage> stages = {};
 
 		bool operator<(const ShaderSpecializationConstant &p_other) const { return constant_id < p_other.constant_id; }
 	};
 
-	struct ShaderDescription {
+	struct ShaderReflection {
 		uint64_t vertex_input_mask = 0;
 		uint32_t fragment_output_mask = 0;
 		bool is_compute = false;
+		bool has_multiview = false;
+		bool has_dynamic_buffers = false;
 		uint32_t compute_local_size[3] = {};
 		uint32_t push_constant_size = 0;
 
 		Vector<Vector<ShaderUniform>> uniform_sets;
 		Vector<ShaderSpecializationConstant> specialization_constants;
-		Vector<ShaderStage> stages;
-	};
-
-protected:
-	struct ShaderReflection : public ShaderDescription {
-		BitField<ShaderStage> stages;
-		BitField<ShaderStage> push_constant_stages;
+		Vector<ShaderStage> stages_vector;
+		BitField<ShaderStage> stages_bits = {};
+		BitField<ShaderStage> push_constant_stages = {};
 	};
 };
-
-#endif // RENDERING_DEVICE_COMMONS_H
