@@ -32,7 +32,7 @@
 
 #include "scene/3d/mesh_instance_3d.h"
 #include "scene/resources/3d/primitive_meshes.h"
-#include "servers/navigation_server_3d.h"
+#include "servers/navigation_3d/navigation_server_3d.h"
 
 namespace TestNavigationServer3D {
 
@@ -95,12 +95,12 @@ TEST_SUITE("[Navigation3D]") {
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->get_process_info(NavigationServer3D::INFO_AGENT_COUNT), 1);
 			navigation_server->agent_set_map(agent, RID());
-			navigation_server->free(map);
+			navigation_server->free_rid(map);
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->get_process_info(NavigationServer3D::INFO_AGENT_COUNT), 0);
 		}
 
-		navigation_server->free(agent);
+		navigation_server->free_rid(agent);
 	}
 
 	TEST_CASE("[NavigationServer3D] Server should manage map properly") {
@@ -172,7 +172,7 @@ TEST_SUITE("[Navigation3D]") {
 			navigation_server->agent_set_map(agent, map);
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->map_get_agents(map).size(), 1);
-			navigation_server->free(agent);
+			navigation_server->free_rid(agent);
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->map_get_agents(map).size(), 0);
 		}
@@ -183,7 +183,7 @@ TEST_SUITE("[Navigation3D]") {
 			navigation_server->link_set_map(link, map);
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->map_get_links(map).size(), 1);
-			navigation_server->free(link);
+			navigation_server->free_rid(link);
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->map_get_links(map).size(), 0);
 		}
@@ -194,7 +194,7 @@ TEST_SUITE("[Navigation3D]") {
 			navigation_server->obstacle_set_map(obstacle, map);
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->map_get_obstacles(map).size(), 1);
-			navigation_server->free(obstacle);
+			navigation_server->free_rid(obstacle);
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->map_get_obstacles(map).size(), 0);
 		}
@@ -205,7 +205,7 @@ TEST_SUITE("[Navigation3D]") {
 			navigation_server->region_set_map(region, map);
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->map_get_regions(map).size(), 1);
-			navigation_server->free(region);
+			navigation_server->free_rid(region);
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->map_get_regions(map).size(), 0);
 		}
@@ -239,7 +239,7 @@ TEST_SUITE("[Navigation3D]") {
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 		}
 
-		navigation_server->free(map);
+		navigation_server->free_rid(map);
 		navigation_server->physics_process(0.0); // Give server some cycles to actually remove map.
 		CHECK_EQ(navigation_server->get_maps().size(), 0);
 	}
@@ -282,12 +282,12 @@ TEST_SUITE("[Navigation3D]") {
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->get_process_info(NavigationServer3D::INFO_LINK_COUNT), 1);
 			navigation_server->link_set_map(link, RID());
-			navigation_server->free(map);
+			navigation_server->free_rid(map);
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->get_process_info(NavigationServer3D::INFO_LINK_COUNT), 0);
 		}
 
-		navigation_server->free(link);
+		navigation_server->free_rid(link);
 	}
 
 	TEST_CASE("[NavigationServer3D] Server should manage obstacles properly") {
@@ -298,7 +298,7 @@ TEST_SUITE("[Navigation3D]") {
 
 		// TODO: Add tests for setters/getters once getters are added.
 
-		navigation_server->free(obstacle);
+		navigation_server->free_rid(obstacle);
 	}
 
 	TEST_CASE("[NavigationServer3D] Server should manage regions properly") {
@@ -335,7 +335,7 @@ TEST_SUITE("[Navigation3D]") {
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->get_process_info(NavigationServer3D::INFO_REGION_COUNT), 1);
 			navigation_server->region_set_map(region, RID());
-			navigation_server->free(map);
+			navigation_server->free_rid(map);
 			navigation_server->physics_process(0.0); // Give server some cycles to commit.
 			CHECK_EQ(navigation_server->get_process_info(NavigationServer3D::INFO_REGION_COUNT), 0);
 		}
@@ -348,7 +348,7 @@ TEST_SUITE("[Navigation3D]") {
 			ERR_PRINT_ON;
 		}
 
-		navigation_server->free(region);
+		navigation_server->free_rid(region);
 	}
 
 	// This test case does not check precise values on purpose - to not be too sensitivte.
@@ -369,8 +369,8 @@ TEST_SUITE("[Navigation3D]") {
 		CHECK_EQ(agent_avoidance_callback_mock.function1_calls, 1);
 		CHECK_NE(agent_avoidance_callback_mock.function1_latest_arg0, Vector3(0, 0, 0));
 
-		navigation_server->free(agent);
-		navigation_server->free(map);
+		navigation_server->free_rid(agent);
+		navigation_server->free_rid(map);
 	}
 
 	// This test case does not check precise values on purpose - to not be too sensitivte.
@@ -411,9 +411,9 @@ TEST_SUITE("[Navigation3D]") {
 		CHECK_MESSAGE(agent_1_safe_velocity.z < 0, "agent 1 should move a bit to the side so that it avoids agent 2");
 		CHECK_MESSAGE(agent_2_safe_velocity.z > 0, "agent 2 should move a bit to the side so that it avoids agent 1");
 
-		navigation_server->free(agent_2);
-		navigation_server->free(agent_1);
-		navigation_server->free(map);
+		navigation_server->free_rid(agent_2);
+		navigation_server->free_rid(agent_1);
+		navigation_server->free_rid(map);
 	}
 
 	TEST_CASE("[NavigationServer3D] Server should make agents avoid dynamic obstacles when avoidance enabled") {
@@ -445,9 +445,9 @@ TEST_SUITE("[Navigation3D]") {
 		CHECK_MESSAGE(agent_1_safe_velocity.x > 0, "Agent 1 should move a bit along desired velocity (+X).");
 		CHECK_MESSAGE(agent_1_safe_velocity.z < 0, "Agent 1 should move a bit to the side so that it avoids obstacle.");
 
-		navigation_server->free(obstacle_1);
-		navigation_server->free(agent_1);
-		navigation_server->free(map);
+		navigation_server->free_rid(obstacle_1);
+		navigation_server->free_rid(agent_1);
+		navigation_server->free_rid(map);
 		navigation_server->physics_process(0.0); // Give server some cycles to commit.
 	}
 
@@ -508,10 +508,10 @@ TEST_SUITE("[Navigation3D]") {
 		CHECK_MESSAGE(agent_2_safe_velocity.x > 0, "Agent 2 should move a bit along desired velocity (+X).");
 		CHECK_MESSAGE(agent_2_safe_velocity.z == 0, "Agent 2 should not move to the side.");
 
-		navigation_server->free(obstacle_1);
-		navigation_server->free(agent_2);
-		navigation_server->free(agent_1);
-		navigation_server->free(map);
+		navigation_server->free_rid(obstacle_1);
+		navigation_server->free_rid(agent_2);
+		navigation_server->free_rid(agent_1);
+		navigation_server->free_rid(map);
 		navigation_server->physics_process(0.0); // Give server some cycles to commit.
 	}
 
@@ -561,8 +561,8 @@ TEST_SUITE("[Navigation3D]") {
 			CHECK_NE(navigation_server->map_get_closest_point(map, Vector3(0, 0, 0)), Vector3(0, 0, 0));
 		}
 
-		navigation_server->free(region);
-		navigation_server->free(map);
+		navigation_server->free_rid(region);
+		navigation_server->free_rid(map);
 		navigation_server->physics_process(0.0); // Give server some cycles to commit.
 		memdelete(mesh_instance);
 		memdelete(node_3d);
@@ -658,8 +658,8 @@ TEST_SUITE("[Navigation3D]") {
 			CHECK_NE(navigation_server->map_get_closest_point(map, Vector3(0, 0, 0)), Vector3(0, 0, 0));
 		}
 
-		navigation_server->free(region);
-		navigation_server->free(map);
+		navigation_server->free_rid(region);
+		navigation_server->free_rid(map);
 		navigation_server->physics_process(0.0); // Give server some cycles to commit.
 		memdelete(mesh_instance);
 		memdelete(node_3d);
@@ -798,8 +798,8 @@ TEST_SUITE("[Navigation3D]") {
 			CHECK_EQ(query_result->get_path().size(), 0);
 		}
 
-		navigation_server->free(region);
-		navigation_server->free(map);
+		navigation_server->free_rid(region);
+		navigation_server->free_rid(map);
 		navigation_server->physics_process(0.0); // Give server some cycles to commit.
 	}
 
